//
// Copyright (c) 2024, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Diagramming;
using MindFusion.Diagramming.WinForms;
using MindFusion.Drawing;

using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.CustomPaging
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		SizeF pageSize = new SizeF(150, 100);
		int pageCols = 1;
		int pageRows = 1;

		public MainForm()
		{
			InitializeComponent();
		}

		void Form1_Load(object sender, EventArgs e)
		{
			diagram.ExteriorBrush = new SolidBrush(Color.Gray);
			diagram.AutoResize = AutoResize.None;
			diagram.Bounds = new RectangleF(new PointF(), pageSize);
			diagram.RestrictItemsToBounds = RestrictToBounds.NoRestriction;

			var node = diagram.Factory.CreateShapeNode(
				30, 30, 30, 30).Selected = true;

			diagram.NodeCreating += OnNodeCreating;
			diagram.NodeModifying += OnNodeModifying;
			diagram.DrawForeground += OnDrawForeground;
		}

		void ResizePageGrid(DiagramNode newNode = null)
		{
			var contentBounds = diagram.GetContentBounds(false, true);
			if (newNode != null)
			{
				contentBounds = RectangleF.Union(
					contentBounds, newNode.Bounds);
			}

			int newPageCols = 1 + (int)(contentBounds.Right / pageSize.Width);
			int newPageRows = 1 + (int)(contentBounds.Bottom / pageSize.Height);

			if (pageCols != newPageCols || pageRows != newPageRows)
			{
				pageCols = newPageCols;
				pageRows = newPageRows;

				diagram.Bounds = new RectangleF(
					0, 0, pageSize.Width * pageCols, pageSize.Height * pageRows);

				diagramView.RecreateCacheImage();
				diagramView.Refresh();
			}
		}

		void OnNodeCreating(object sender, NodeValidationEventArgs e)
		{
			ResizePageGrid(e.Node);
		}

		void OnNodeModifying(object sender, NodeValidationEventArgs e)
		{
			ResizePageGrid();
		}

		void OnDrawForeground(object sender, DiagramEventArgs e)
		{
			var pen = new System.Drawing.Pen(Color.Black)
			{
				Width = 0,
				DashStyle = System.Drawing.Drawing2D.DashStyle.Dot
			};

			// draw vertical page borders
			for (int c = 0; c <= pageCols; c++)
			{
				float x = c * pageSize.Width;
				e.Graphics.DrawLine(
					pen, x, 0, x, diagram.Bounds.Height);
			}

			// draw horizontal page borders
			for (int r = 0; r <= pageRows; r++)
			{
				float y = r * pageSize.Height;
				e.Graphics.DrawLine(
					pen, 0, y, diagram.Bounds.Width, y);
			}

			pen.Dispose();
		}
	}
}
